function t = MissRatio_erttreetrain(Xdata,param,liv)
% t = MissRatio_erttreetrain(Xdata,param,liv)
%  
% train a tree with missing data, adapting the 
% scheme of Extremely Randomized Trees
% P. Geurts et al: "Extremely randomized trees",
% Mach. Learn., 63 (1), 3–42, 2006.
%
% In particular, when a missing value is encountered
% there are 3 options (default: param.misstrain = 2) 
% 1. propagate points with NaN in both directions
% 2. propagate points with NaN in one random direction
% 3. stop points with NaN
% 
% Miss RatioRF Random Forest library
% (c) Manuele Bicego 2025
% 
% Xdata: [nxd]: n objects in a d-dimensional space
%        Nan indicates the presence of a missing value
%

global idn

n = size(Xdata,1);
t.idn = idn;
t.liv = liv;
allf = param.allf;

% criterion at node
stopsplit = 0;
bestt = nan;
mi = isnan(Xdata);
miXdata = Xdata;
miXdata(mi) = max(Xdata(:))+1; % a value not present
% miXdata: data in which we replace nan with max+1 
% (for checks related to identical values)
if sum(mi==0,'all') == 0
    % all NaN
    stopsplit = 1;
elseif liv > param.maxDepth
    % max length
    stopsplit = 1;
elseif n == 1
    % leave with one element
    stopsplit = 1;
elseif sum(std(miXdata)>=1e-10) == 0
    % all elements equal
    stopsplit = 1;
else
    % random feature
    % choose among a feature for which:
    %   i) we have different values
    %   ii) we do not have only nan
    %   iii) we have more than one value
    %   iv) it is in the set allf
    buoni = sort(allf);
    for jj = length(allf):-1:1
        j = buoni(jj);
        xj = miXdata(:,j);
        un = unique(xj);
        un(find(un == max(Xdata(:))+1)) = [];
        if (std(xj)<=1e-10 || mean(mi(:,j))==1 || length(un) == 1)
            buoni(jj) = [];
        end
    end
    if isempty(buoni)
        stopsplit = 1;
    else
        a = randi(length(buoni));
        bestf = buoni(a);
        % random pick of splitting value in the domain of non Nan values
        values = Xdata(:,bestf);
        values(mi(:,bestf)) = [];
        xi = sort(values);
        th = xi(1) + (xi(end) - xi(1))*rand;
        bestt = th;
        
        % decide how to propagate points with missing data
        mi = mi(:,bestf);
        nomiXdata=Xdata;
        nomiXdata(mi==1,:)=[];
        a = find(nomiXdata(:,bestf)>th);
        if isempty(a)
            % th is exactly the last element
            a = find(nomiXdata(:,bestf)>=th);
        end
        XdataL = nomiXdata;
        XdataL(a,:)=[];
        XdataR = nomiXdata(a,:);
        % Management of missingdata
        switch param.misstrain
            case 1
                % 1. propagate points with NaN in both directions
                XdataL = [XdataL;Xdata(mi==1,:)];
                XdataR = [XdataR;Xdata(mi==1,:)];
            case 2
                % 2. propagate points with NaN in one random direction
                b = randi(2);
                if b == 1
                    XdataL = [XdataL;Xdata(mi==1,:)];
                else
                    XdataR = [XdataR;Xdata(mi==1,:)];
                end
            case 3
                % 3. stop points with NaN
                % nothing to do
        end
    end
end
%[idn bestt]
if stopsplit
    % leave
    t.l = [];
    t.r = [];
    t.sizet = n;
    t.nleaves = 1;
    t.maxdepth = liv;
    t.Mgain = 0;
    t.nInternals = 0;
    t.maxidn = idn;
else
    t.bestf = bestf;
    t.bestt = bestt;
    t.Mgain = 1;
    idn = idn+1;
    t.l = MissRatio_erttreetrain(XdataL,param,liv+1);
    idn = idn+1;
    t.r = MissRatio_erttreetrain(XdataR,param,liv+1);
    t.sizet = n;
    t.nleaves = t.l.nleaves + t.r.nleaves; % due foglie, una radice
    t.maxdepth = max(t.l.maxdepth,t.r.maxdepth);
    t.nInternals = t.l.nInternals + t.r.nInternals +1;
    t.maxidn = max( t.l.maxidn, t.r.maxidn);
end

