function t = MissRatio_regtreetrain(Xdata,param,liv)
% t = MissRatio_regtreetrain(Xdata,param,liv)
%  
% train a tree with missing data, adapting the 
% scheme of Regression Trees: choose one feature to regress
% and one feature to use as predictor
%
% In particular, when a missing value is encountered
% there are 3 options (default: param.misstrain = 2) 
% 1. propagate points with NaN in both directions
% 2. propagate points with NaN in one random direction
% 3. stop points with NaN
% 
% Miss RatioRF Random Forest library
% (c) Manuele Bicego 2025
% 
% Xdata: [nxd]: n objects in a d-dimensional space
%        Nan indicates the presence of a missing value
%

global idn

n = size(Xdata,1);
t.idn = idn;
t.liv = liv;
allf = param.allf;
if param.featsubset == 0
    featsubset = length(param.allf);
else
    featsubset = param.featsubset;
end
fss = randperm(length(allf));
fss = allf(fss(1:featsubset));

stopsplit = 0;
bestt = nan;
mi = isnan(Xdata);
miXdata = Xdata;
miXdata(mi) = max(Xdata(:))+1; % a value not present
% miXdata: data in which we replace nan with max+1 
% (for checks related to identical values)
if sum(mi==0,'all') == 0
    % all NaN
    stopsplit = 1;
elseif liv > param.maxDepth
    % max length
    stopsplit = 1;
elseif n == 1
    % leave with one element
    stopsplit = 1;
elseif sum(std(miXdata)>=1e-10) == 0
    % all elements equal
    stopsplit = 1;
else
    % choose a feature to regress and a feature to use as predictor
    % choose among a feature for which:
    %   i) we have different values
    %   ii) we do not have only nan
    %   iii) we have more than one value
    %   iv) it is in the set allf
    buoni = sort(allf);
    for jj = length(allf):-1:1
        j = buoni(jj);
        xj = miXdata(:,j);
        un = unique(xj);
        un(find(un == max(Xdata(:))+1)) = [];
        if (std(xj)<=1e-10 || mean(mi(:,j))==1 || length(un) == 1)
            buoni(jj) = [];
        end
    end
    if isempty(buoni) | length(buoni) == 1
        % not enough feature to perferom regression
        stopsplit = 1;
    else
        a = randperm(length(buoni));
        targe = buoni(a(1));
        bestf = buoni(a(2));
        
        values = Xdata(:,bestf);
        yvalues = Xdata(:,targe);
        mimi = mi(:,bestf) | mi(:,targe);
        values(mimi) = [];
        yvalues(mimi) = [];
        
        if isempty(values) | length(unique(values)) == 1
            % not enough values
            stopsplit = 1; 
        else
            % criterion at node
            Rt = MissRatio_regressiontreevalue(yvalues);
            bestdiff = -inf;
            % sort the data along feature bestf:
            [xi,I] = sort(values);
            tmpy = yvalues(I);
            for j=1:length(values)-1
                splitest = (xi(j) ~= xi(j+1));
                % skip equal points
                if splitest
                    RtL = MissRatio_regressiontreevalue(tmpy(1:j));
                    nl = length([1:j]);
                    RtR = MissRatio_regressiontreevalue(tmpy(j+1:end));
                    nr = length([j+1:n]);
                    %diff = Rt - nr*RtR - nl*RtL;
                    diff = Rt - nr*RtR/(nr+nl) - nl*RtL/(nr+nl);
                    if diff > bestdiff
                        bestdiff = diff;
                        bestt = mean(xi(j:j+1));
                    end
                end
            end
        end
        
        % decide how to propagate points with missing data
        mi = mi(:,bestf);
        nomiXdata=Xdata;
        nomiXdata(mi==1,:)=[];
        a = find(nomiXdata(:,bestf)>bestt);
        if isempty(a)
            % th is exactly the last element
            a = find(nomiXdata(:,bestf)>=bestt);
        end
        XdataL = nomiXdata;
        XdataL(a,:)=[];
        XdataR = nomiXdata(a,:);
        % Management of missingdata
        switch param.misstrain
            case 1
                % 1. propagate points with NaN in both directions
                XdataL = [XdataL;Xdata(mi==1,:)];
                XdataR = [XdataR;Xdata(mi==1,:)];
            case 2
                % 2. propagate points with NaN in one random direction
                b = randi(2);
                if b == 1
                    XdataL = [XdataL;Xdata(mi==1,:)];
                else
                    XdataR = [XdataR;Xdata(mi==1,:)];
                end
            case 3
                % 3. stop points with NaN
                % nothing to do
        end
    end
end
%[idn bestt]
if stopsplit
    % leave
    t.l = [];
    t.r = [];
    t.sizet = n;
    t.nleaves = 1;
    t.maxdepth = liv;
    t.Mgain = 0;
    t.nInternals = 0;
    t.maxidn = idn;
else
    t.bestf = bestf;
    t.bestt = bestt;
    t.Mgain = 1;
    idn = idn+1;
    t.l = MissRatio_regtreetrain(XdataL,param,liv+1);
    idn = idn+1;
    t.r = MissRatio_regtreetrain(XdataR,param,liv+1);
    t.sizet = n;
    t.nleaves = t.l.nleaves + t.r.nleaves; % due foglie, una radice
    t.maxdepth = max(t.l.maxdepth,t.r.maxdepth);
    t.nInternals = t.l.nInternals + t.r.nInternals +1;
    t.maxidn = max( t.l.maxidn, t.r.maxidn);
end

