% Demo for MissRatioRF distance, a Random Forest distance 
% which can be computed in presence of missing data
% 
% More info:
% M. Bicego, F. Cicalese: "Computing Random Forest-distances 
% in the presence of missing data", ACM Transactions on Knowledge 
% Discovery from Data, vol. 18(7), pp. 1-18, (2024)
%
% (c) Manuele Bicego 
% Last update: Jan 2025

clear all
close all

addpath('./MissingRatioRFLib')


% Test on the iris dataset
load('iris.mat');

% Injecting missing data, using protocol MCAR
missingrate = 0.1;
missdata = createMCAR(data,missingrate);


% Parameter for MissRatio RF distance
mparam = MissRatio_defaultParam;

%%%%% Congigurations tested in the paper

%%% CONF 1: ERT training, NanStop, growing trees until maxdepth
mparam.misstrain = 3;
mparam.maxDepth = Inf;
mparam.typeTree =  2; % ERT

%%% CONF 2: ERT training, NanBoth, growing trees until logn
% mparam.misstrain = 1;
% mparam.maxDepth = 0;
% mparam.typeTree =  2; % ERT

% For both configurations
% For datasets larger that fastthr, subsample more
FASTthr = 400;
FASTsampling = 128;           


if size(data,1) >= FASTthr
    mparam.sampling = FASTsampling;
end

fprintf('Training Forest');
mrfc = MissRatio_RFtrain(missdata,mparam);
fprintf('...done\n');

fprintf('Computing MissRatioRF distances');
CI = MissRatio_getRFPathInfo(missdata,mrfc,mparam);
[Dmissing1,Dmissing2] = MissRatio_getRFDist_v2(missdata,mrfc,mparam,CI);
fprintf('...done\n');


% Dmissing1,Dmissing2 are the two MissRatioRF variants (equations 9-10 of the paper)
% 
% From our paper:
% Concerning the difference between the two variants, we can observe that for low levels of missingness 
% the two distances are more or less equivalent. When considering larger values of missingness, two different
% behaviours appear: on the datasets of small scale, there is not a predominance of one of the two variants.
% On the contrary, on datasets of larger scale MissRatioRF 2 is always better than the counterpart. 
% This seems to be reasonable: on these sets each tree is built with a small fraction of the objects (128), and this
% fact probably induces more diverse trees with respect to the case of small datasets, in which each tree is built
% with 50% of the dataset. MissRatioRF 2 is able to explicitly consider and quantify the diversity between the
% trees: considering that this diversity is drastically amplified by the mechanisms of missingness management
% we employed, it seems reasonable to note a more remarkable improvement with large levels of missingness.



% Comparison with the RatioRF measure computed on the whole data
% Compute the RatioRF distance on the complete data
% Requires the Random Forest Clustering library, available here:
% http://profs.sci.univr.it/~bicego/code.html
% (a zip is also included here)

complete = data;
param = RFC_defaultParam;
param.dist = 5; % RatioRF
param.typeTree = 2; % ERT
if size(data,1) >= FASTthr
    param.sampling = FASTsampling;
end
rfc = RFC_RFtrain(complete,param);
CI = RFC_getRFClusInfo(complete,rfc,param);
Dcomplete = RFC_getRFDist(complete,rfc,param,CI);

% Compute the difference
figure, 
subplot(2,3,1)
imagesc(Dcomplete), title ('RatioRF on complete data ')
colorbar

subplot(2,3,2)
imagesc(Dmissing1), title ('MissRatioRF v1')
colorbar

subplot(2,3,3)
imagesc(Dmissing2), title ('MissRatioRF v2')
colorbar

 
subplot(2,3,5)
imagesc(abs(Dcomplete-Dmissing1)), title ('Difference (MissRatioRF v1)')
colorbar

subplot(2,3,6)
imagesc(abs(Dcomplete-Dmissing2)), title ('Difference (MissRatioRF v2)')
colorbar




